###################################################################################################
# _filtermanager.py
#
# $Id$
# $Name$
# $Author$
# $Revision$
#
# Implementation of class FilterManager (see below).
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from __future__ import nested_scopes
from Globals import HTMLFile
import ZPublisher.HTTPRequest
import copy
import urllib
# Product Imports.
import _blobfields
import _globals


"""
################################################################################################
#
#   P R O C E S S E S
#
################################################################################################
"""

# ----------------------------------------------------------------------------------------------
#  _filtermanager.getRawProcesses:
#
#  Returns raw dictionary of processes.
# ----------------------------------------------------------------------------------------------
def getRawProcesses(self):
  # Return attribute.
  return self.getConfProperty('ZMS.filter.processes',{})

# ----------------------------------------------------------------------------------------------
#  _filtermanager.setProcess:
# 
#  Set/add process specified by given Id.
# ----------------------------------------------------------------------------------------------
def setProcess(self, newId, newAcquired=0, newName='', newCommand=''):
  # Set.
  obs = getRawProcesses(self)
  ob = {}
  ob['acquired'] = newAcquired
  ob['name'] = newName
  ob['command'] = newCommand
  obs[newId] = ob
  # Set attribute.
  self.setConfProperty('ZMS.filter.processes',obs.copy())
  # Return with new id.
  return newId

# ----------------------------------------------------------------------------------------------
#  _filtermanager.delProcess:
# 
#  Delete process specified by given Id.
# ----------------------------------------------------------------------------------------------
def delProcess(self, id):
  # Delete.
  cp = getRawProcesses(self)
  obs = {}
  for key in cp.keys():
    if key != id:
      obs[key] = cp[key]
  # Set attribute.
  self.setConfProperty('ZMS.filter.processes',obs.copy())
  # Return with empty id.
  return ''


"""
################################################################################################
#
#   F I L T E R S
#
################################################################################################
"""

# ----------------------------------------------------------------------------------------------
#  _filtermanager.getRawFilters:
#
#  Returns raw dictionary of filters.
# ----------------------------------------------------------------------------------------------
def getRawFilters(self):
  # Return attribute.
  return self.getConfProperty('ZMS.filter.filters',{})

# ----------------------------------------------------------------------------------------------
#  _filtermanager.setFilter:
# 
#  Set/add filter specified by given Id.
# ----------------------------------------------------------------------------------------------
def setFilter(self, newId, newAcquired=0, newName='', newFormat=''):
  # Set.
  obs = getRawFilters(self)
  ob = {}
  ob['acquired'] = newAcquired
  ob['name'] = newName
  ob['format'] = newFormat
  obs[newId] = ob
  # Set attribute.
  self.setConfProperty('ZMS.filter.filters',obs.copy())
  # Return with new id.
  return newId

# ----------------------------------------------------------------------------------------------
#  _filtermanager.delFilter:
# 
#  Delete filter specified by given Id.
# ----------------------------------------------------------------------------------------------
def delFilter(self, id):
  # Delete.
  cp = getRawFilters(self)
  obs = {}
  for key in cp.keys():
    if key != id:
      obs[key] = cp[key]
  # Set attribute.
  self.setConfProperty('ZMS.filter.filters',obs.copy())
  # Return with empty id.
  return ''


"""
################################################################################################
#
#   F I L T E R - P R O C E S S E S
#
################################################################################################
"""

# ----------------------------------------------------------------------------------------------
#  _filtermanager.setFilterProcess:
# 
#  Set/add filter-process specified by given Ids.
# ----------------------------------------------------------------------------------------------
def setFilterProcess(self, id, newProcessId, newProcessFile=None):
  # Set.
  obs = getRawFilters(self)
  ob = {}
  ob['id'] = newProcessId
  ob['file'] = newProcessFile
  pobs = obs[id].get('processes',[])
  pobs.append(ob)
  obs[id]['processes'] = pobs
  # Set attribute.
  self.setConfProperty('ZMS.filter.filters',obs.copy())
  # Return with new id.
  return len(pobs)-1

# ----------------------------------------------------------------------------------------------
#  _filtermanager.delFilterProcess:
# 
#  Delete filter-process specified by given Ids.
# ----------------------------------------------------------------------------------------------
def delFilterProcess(self, id, pid):
  # Delete.
  obs = getRawFilters(self)
  pobs = obs[id].get('processes',[])
  pobs.remove(pobs[pid])
  obs[id]['processes'] = pobs
  # Set attribute.
  self.setConfProperty('ZMS.filter.filters',obs.copy())
  # Return with empty id.
  return -1

# ----------------------------------------------------------------------------------------------
#  _filtermanager.moveFilterProcess:
# 
#  Move filter-process specified by given Ids in specified direction (-1: up, +1: down).
# ----------------------------------------------------------------------------------------------
def moveFilterProcess(self, id, pid, dir):
  pass

################################################################################################
################################################################################################
###
###   class FilterManager
###
################################################################################################
################################################################################################
class FilterManager: 

    # Management Interface.
    # ---------------------
    manage_customizeFilterForm = HTMLFile('dtml/zms/manage_customizefilterform', globals())


    # ------------------------------------------------------------------------------------------
    #  FilterManager.getProcessIds:
    # 
    #  Returns list of process-Ids.
    # ------------------------------------------------------------------------------------------
    def getProcessIds(self, sort=1):
      obs = getRawProcesses(self)
      ids = obs.keys()
      if sort:
        mapping = map(lambda x: (self.getProcess(x)['name'],x),ids)
        mapping.sort()
        ids = map(lambda x: x[1],mapping)
      return ids

    # ------------------------------------------------------------------------------------------
    #  FilterManager.getProcess:
    # 
    #  Returns process specified by Id.
    # ------------------------------------------------------------------------------------------
    def getProcess(self, id):
      ob = getRawProcesses(self)[id].copy()
      ob['id'] = id
      # Acquire from parent.
      if ob.get('acquired',0):
        masterHome = getattr(self.getHome(),self.getConfProperty('Portal.Master'))
        masterDocElmnt = masterHome.objectValues(['ZMS'])[0]
        ob = masterDocElmnt.getProcess(id)
        ob['acquired'] = 1
      return ob


    # ------------------------------------------------------------------------------------------
    #  FilterManager.getFilterIds:
    # 
    #  Returns list of filter-Ids.
    # ------------------------------------------------------------------------------------------
    def getFilterIds(self, sort=1):
      obs = getRawFilters(self)
      ids = obs.keys()
      if sort:
        mapping = map(lambda x: (self.getFilter(x)['name'],x),ids)
        mapping.sort()
        ids = map(lambda x: x[1],mapping)
      return ids

    # ------------------------------------------------------------------------------------------
    #  FilterManager.getFilter:
    # 
    #  Returns filter specified by Id.
    # ------------------------------------------------------------------------------------------
    def getFilter(self, id):
      ob = getRawFilters(self)[id].copy()
      ob['id'] = id
      # Acquire from parent.
      if ob.get('acquired',0):
        masterHome = getattr(self.getHome(),self.getConfProperty('Portal.Master'))
        masterDocElmnt = masterHome.objectValues(['ZMS'])[0]
        ob = masterDocElmnt.getFilter(id)
        ob['acquired'] = 1
      return ob

    # ------------------------------------------------------------------------------------------
    #  FilterManager.getFilterProcesses:
    # 
    #  Returns list of processes for filter specified by Id.
    # ------------------------------------------------------------------------------------------
    def getFilterProcesses(self, id):
      ob = self.getFilter(id).get('processes',[])
      return ob


    ###############################################################################################
    #  FilterManager.manage_changeFilter:
    #
    #  Customize filter.
    ###############################################################################################
    def manage_changeFilter(self, lang, manage_lang, btn='', key='', REQUEST=None, RESPONSE=None):
      """ FilterManager.manage_changeFilter """
      message = ''
      id = REQUEST.get('id','')
      pid = REQUEST.get('pid',-1)

      # Acquire.
      # --------
      if btn == self.getLangStr('BTN_ACQUIRE',manage_lang):
        newId = REQUEST.get('aq_id')
        newAcquired = 1
        id = setFilter(self, newId, newAcquired)
        message = self.getLangStr('MSG_INSERTED',manage_lang)%id

      # Change.
      # -------
      elif btn == self.getLangStr('BTN_CHANGE',manage_lang):
        if key == 'obj':
          newId = REQUEST.get('inpId').strip()
          newAcquired = 0
          newName = REQUEST.get('inpName').strip()
          newFormat = REQUEST.get('inpFormat').strip()
          id = delFilter(self, id)
          id = setFilter(self, newId, newAcquired, newName, newFormat)
        elif key == 'attr':
          pass
        message = self.getLangStr('MSG_CHANGED',manage_lang)

      # Delete.
      # -------
      elif btn == self.getLangStr('BTN_DELETE',manage_lang) and key == 'obj':
        id = delFilter(self, id)
        message = self.getLangStr('MSG_DELETED',manage_lang)%int(1)
      elif btn == 'delete' and key == 'attr':
        pid = delFilterProcess(self, id, pid)
        message = self.getLangStr('MSG_DELETED',manage_lang)%int(1)

      # Export.
      # -------
      elif btn == self.getLangStr('BTN_DELETE',manage_lang):
        pass

      # Import.
      # -------
      elif btn == self.getLangStr('BTN_IMPORT',manage_lang):
        pass

      # Insert.
      # -------
      elif btn == self.getLangStr('BTN_INSERT',manage_lang):
        if key == 'obj':
          newId = REQUEST.get('newId').strip()
          newAcquired = 0
          newName = REQUEST.get('newName').strip()
          newFormat = REQUEST.get('newFormat').strip()
          id = setFilter(self, newId, newAcquired, newName, newFormat)
          message = self.getLangStr('MSG_INSERTED',manage_lang)%id
        elif key == 'attr':
          newProcessId = REQUEST.get('newFilterProcessId')
          newProcessFile = REQUEST.get('newFilterProcessFile')
          if getattr(newProcessFile, '__class__',None) is ZPublisher.HTTPRequest.FileUpload:
            if len(getattr(newProcessFile, 'filename',''))==0:
              newProcessFile = None
            else:
              newProcessFile = _blobfields.createBlobField(self, _globals.DT_FILE, newProcessFile)
          pid = setFilterProcess(self, id, newProcessId, newProcessFile)
          message = self.getLangStr('MSG_INSERTED',manage_lang)%newProcessId

      # Move down.
      # --------
      elif btn == 'move_down':
        moveFilterProcess(self, id, pid, +1)
        message = self.getLangStr('MSG_MOVEDOBJDOWN', manage_lang)%("<i>%s</i>"%pid)

      # Move up.
      # --------
      elif btn == 'move_up':
        moveFilterProcess(self, id, pid, -1)
        message = self.getLangStr('MSG_MOVEDOBJUP', manage_lang)%("<i>%s</i>"%pid)

      # Return with message.
      message = urllib.quote(message)
      return RESPONSE.redirect('manage_customizeFilterForm?id=%s&pid:int=%i&lang=%s&manage_lang=%s&manage_tabs_message=%s'%(id,pid,lang,manage_lang,message))


    ###############################################################################################
    #  FilterManager.manage_changeProcess:
    #
    #  Customize process.
    ###############################################################################################
    def manage_changeProcess(self, lang, manage_lang, btn='', key='', REQUEST=None, RESPONSE=None):
      """ FilterManager.manage_changeProcess """
      message = ''
      id = REQUEST.get('id','')

      # Acquire.
      # --------
      if btn == self.getLangStr('BTN_ACQUIRE',manage_lang):
        newId = REQUEST.get('aq_id')
        newAcquired = 1
        id = setProcess(self, newId, newAcquired)
        message = self.getLangStr('MSG_INSERTED',manage_lang)%id

      # Change.
      # -------
      elif btn == self.getLangStr('BTN_CHANGE',manage_lang):
        newId = REQUEST.get('inpId').strip()
        newAcquired = 0
        newName = REQUEST.get('inpName').strip()
        newCommand = REQUEST.get('inpCommand').strip()
        id = delProcess(self, id)
        id = setProcess(self, newId, newAcquired, newName, newCommand)
        message = self.getLangStr('MSG_CHANGED',manage_lang)

      # Delete.
      # -------
      elif btn == self.getLangStr('BTN_DELETE',manage_lang):
        id = delProcess(self, id)
        message = self.getLangStr('MSG_DELETED',manage_lang)%int(1)

      # Export.
      # -------
      elif btn == self.getLangStr('BTN_DELETE',manage_lang):
        pass

      # Import.
      # -------
      elif btn == self.getLangStr('BTN_IMPORT',manage_lang):
        pass

      # Insert.
      # -------
      elif btn == self.getLangStr('BTN_INSERT',manage_lang):
        newId = REQUEST.get('newId').strip()
        newAcquired = 0
        newName = REQUEST.get('newName').strip()
        newCommand = REQUEST.get('newCommand').strip()
        id = setProcess(self, newId, newAcquired, newName, newCommand)
        message = self.getLangStr('MSG_INSERTED',manage_lang)%id

      # Return with message.
      message = urllib.quote(message)
      return RESPONSE.redirect('manage_customizeFilterForm?id=%s&lang=%s&manage_lang=%s&manage_tabs_message=%s'%(id,lang,manage_lang,message))

###################################################################################################
