###################################################################################################
# _htmldoc.py
#
# $Id: _htmldoc.py,v 1.1 2003/08/31 13:29:22 dnordmann Exp $
# $Name:  $
# $Author: dnordmann $
# $Revision: 1.1 $
#
# Implementation of class Exportable (see below).
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
import tempfile
import os
# Product Imports.
import _fileutil
import _globals


# ---------------------------------------------------------------------------------------------
#  _htmldoc.html2pdfbook:
# ---------------------------------------------------------------------------------------------
def html2pdfbook(self, file, zip_result, REQUEST):

  # Create temporary folder.
  # ------------------------
  folder = tempfile.mktemp()
  os.mkdir(folder)
  
  # Save to temporary file.
  # -----------------------
  rawfilename = _fileutil.getOSPath('%s/%s'%(folder,_fileutil.extractFileName('printable.zip')))
  _fileutil.exportObj(file,rawfilename)

  # Process PDF-file.
  # -----------------
  f = open(rawfilename,'rb')
  rtn = html2pdf(self=self,file=f,zip_result=zip_result,options=' --book --toctitle "%s" --header t.l'%self.getLangStr('ATTR_TOC',REQUEST['lang']))
  f.close()

  # Remove temporary files.
  # -----------------------
  _fileutil.remove(folder,deep=1)
  
  # Return PDF-file.
  # ----------------
  return rtn


# ---------------------------------------------------------------------------------------------
#  _htmldoc.html2pdf:
# ---------------------------------------------------------------------------------------------
def html2pdf(self, file, zip_result=0, options=''):
    
  # Create temporary folder.
  # ------------------------
  folder = tempfile.mktemp()
  os.mkdir(folder)
  
  # Save to temporary file.
  # -----------------------
  #-- Filename.
  if type(file) is type(''):
    filename = 'printable.html'
  else:
    try: filename = file.name
    except: filename = file.filename
  #-- Save.
  rawfilename = _fileutil.getOSPath('%s/%s'%(folder,_fileutil.extractFileName(filename)))
  _fileutil.exportObj(file,rawfilename)
  #-- Find (in Zip-Archive).
  htmlfilename = None
  if _fileutil.extractFileExt(filename) == 'zip':
    _fileutil.extractZipArchive(rawfilename)
    for ext in ['htm','html']:
      if htmlfilename is None:
        htmlfilename = _fileutil.findExtension(ext,folder,deep=0)
    for ext in ['htm','html']:
      if htmlfilename is None:
        htmlfilename = _fileutil.findExtension(ext,folder,deep=1)
  else:
    htmlfilename = rawfilename
  pdffilename = _fileutil.getOSPath('%s/%s'%(folder,_fileutil.extractFileName('printable.pdf')))
  
  # <HTML>DOC                   
  # http://www.easysw.com/htmldoc
  #
  #   --size <A4>		Page size DIN A4.
  #   --book			Specifies that the input files comprise a book with chapters and heading.
  #   --toctitle <string>	Specifies the String to display at the top of the 
  #				table-of-contents.
  #   --logoimage <filename>	Specifies the Logo image for the HTML navigation bar and
  #				page headers and footers (supported formats are GIF, JPEG
  #				and PNG).
  #   --header lcr		Specifies the contents of the page header.
  #				The lcr parameter is a three-character string representing
  #				the left, center and right header fields.
  #				Each character can be one of the following:
  #					.	Blank
  #					1	Current page number in decimals (1, 2, 3,...)
  #					c	Current chapter title
  #					C	Current chapter page number
  #					h	Current heading
  #					i	Current page number in lowercase romans (i, ii, iii,...)
  #					I	Current page number in uppercase romans (i, ii, iii,...)
  #					l	Logo image
  #					t	Document title
  #   -t <pdf13>		Specifies the output format for the document.
  #   -f <filename>		Specifies an output file for the document.
  #
  # >>> Important Note:
  # >>> DOS, Win32: <HTML>DOC must be installed in the path.
  # >>> UNIX, Linux, Solaris: ...
  command = "htmldoc"
  
  #-- assemble command line
  command = command \
          + ' ' + options \
          + ' --size A4' \
          + ' -t pdf13' \
          + ' -f %s'%(pdffilename) \
          + ' %s'%(htmlfilename)
  #-- Execute command-line.
  os.system(command)
  
  # Import PDF-file.
  # ----------------
  if zip_result:
    rtn = _fileutil.buildZipArchive(pdffilename)
  else:
    f = open(pdffilename,'r')
    rtn = f.read()
    f.close()
  
  # Remove temporary files.
  # -----------------------
  _fileutil.remove(folder,deep=1)
  
  # Return PDF-file.
  # ----------------
  return rtn


###################################################################################################
