###################################################################################################
# zmssqldb.py
#
# $Id: zmssqldb.py,v 1.2 2003/09/23 19:24:50 dnordmann Exp $
# $Name:  $
# $Author: dnordmann $
# $Revision: 1.2 $
#
# Implementation of class ZMSSqlDb (see below).
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from __future__ import nested_scopes
from Globals import HTML, HTMLFile
from Products.ZSQLMethods.SQL import SQLConnectionIDs
import copy
import sys
import urllib
# Product Imports.
from zmsobject import ZMSObject
import _globals
import _objattrs


SQLConnectionIDs = SQLConnectionIDs


# ---------------------------------------------------------------------------------------------
#  _workflowmanager.setSqlView
# ---------------------------------------------------------------------------------------------
def setSqlView(self, id, newId, newSql):
  obs = getattr(self,'sql_views',{})
  if id in obs.keys():
    del obs[id]
  obs[newId] = {'sql':newSql}
  setattr(self,'sql_views',obs.copy())

# ---------------------------------------------------------------------------------------------
#  _workflowmanager.delSqlView
# ---------------------------------------------------------------------------------------------
def delSqlView(self, id):
  obs = getattr(self,'sql_views',{})
  if id in obs.keys():
    del obs[id]
  setattr(self,'sql_views',obs.copy())


###################################################################################################
###################################################################################################
###   
###   C o n s t r u c t o r ( s )
###   
###################################################################################################
###################################################################################################

manage_addZMSSqlDbForm = HTMLFile('manage_addzmssqldbform', globals()) 
def manage_addZMSSqlDb(self, lang, manage_lang, _sort_id, REQUEST, RESPONSE):
  """ manage_addZMSSqlDb """
    
  ##### Create ####
  id_prefix = _globals.id_prefix(REQUEST.get('id','e'))
  obj = ZMSSqlDb(self.getNewId(id_prefix),_sort_id+1)
  self._setObject(obj.id, obj)
  
  obj = getattr(self,obj.id)
  ##### Object State ####
  obj.setObjStateNew(REQUEST)
  ##### Init Properties ####
  obj.manage_changeProperties(lang,manage_lang,REQUEST,RESPONSE)
  ##### VersionManager ####
  obj.onChangeObj(REQUEST)
            
  ##### Normalize Sort-IDs ####
  self.normalizeSortIds(id_prefix)
        
  # Return with message.        
  if REQUEST.RESPONSE:
    message = self.getLangStr('MSG_INSERTED',manage_lang)%obj.display_type(REQUEST)
    REQUEST.RESPONSE.redirect('%s/%s/manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(self.absolute_url(),obj.id,lang,manage_lang,urllib.quote(message)))


###################################################################################################
###################################################################################################
###   
###   C l a s s
###   
###################################################################################################
###################################################################################################

class ZMSSqlDb(ZMSObject):

    # Properties.
    # -----------
    meta_type = "ZMSSqlDb"
    icon = "misc_/zms/zmssqldb.gif"
    icon_disabled = "misc_/zms/zmssqldb.gif"

    # Management Options.
    # -------------------
    manage_options = ( 
	{'label': 'TAB_EDIT',		'action': 'manage_main'},
	{'label': 'TAB_PROPERTIES',	'action': 'manage_properties'},
	)

    # Management Permissions.
    # -----------------------
    __authorPermissions__ = (
		'manage','manage_main','manage_workspace',
		'manage_mainExpert','manage_expertProperties',
		'manage_properties','manage_changeProperties',
		'manage_moveObjUp','manage_moveObjDown',
		'manage_cutObjects','manage_copyObjects','manage_pasteObjs',
		'manage_userForm','manage_user',
		)
    __editorPermissions__ = (
		)
    __administratorPermissions__ = (
		)
    __ac_permissions__=(
		('ZMS Author', __authorPermissions__),
		('ZMS Editor', __editorPermissions__),
		('ZMS Administrator', __administratorPermissions__),
		)

    # Properties.
    # -----------
    __obj_attrs__ = {
        # Changed by
        'change_uid':{'datatype':'string','multilang':1,'xml':0,'lang_inherit':_globals.false},
        'change_dt':{'datatype':'datetime','multilang':1,'xml':0,'lang_inherit':_globals.false},
        # Properties
        'sort_id':{'datatype':'string','xml':0},
    }


    # Management Interface.
    # ---------------------
    manage_main = HTMLFile('dtml/zmssqldb/manage_main', globals())
    manage_mainExpert = HTMLFile('dtml/zmssqldb/manage_mainexpert', globals())
    manage_properties = HTMLFile('dtml/zmssqldb/manage_properties', globals())


    """
    ###############################################################################################    
    #
    #   CONSTRUCTOR
    #
    ###############################################################################################    
    """

    # -------------------------------------------------------------------------------------------------
    # ZMSSqlDb.recurse_updateVersion:
    #
    # Update version.
    # -------------------------------------------------------------------------------------------------
    def recurse_updateVersion(self, REQUEST):
      message = ''

      # Process super-objects.
      message += ZMSObject.recurse_updateVersion(self,REQUEST)
        
      # Process object.
      # [...]

      # Return with message.
      return message


    # ------------------------------------------------------------------------------------------
    #  ZMSSqlDb.sql_quote__:
    # ------------------------------------------------------------------------------------------
    def sql_quote__(self, tablename, key, v):
      attr = self.getEntityAttr(tablename,key)
      if attr['type'] in ['float','int']:
        return "%s" % str(v)
      else:
        v = str(v)
        if v.find("\'") >= 0: v=''.join(v.split("\'"),"''")
        return "'%s'"%v

    # ------------------------------------------------------------------------------------------
    #  ZMSSqlDb.query:
    # ------------------------------------------------------------------------------------------
    def query(self, qs):
      from cStringIO import StringIO
      from Shared.DC.ZRDB.Results import Results
      from Shared.DC.ZRDB import RDB
      databaseAdptr = getattr(self,self.connection_id)
      dbc = databaseAdptr._v_database_connection
      res = dbc.query(qs)
      if type(res) is type(''):
        f=StringIO()
        f.write(res)
        f.seek(0)
        result=RDB.File(f)
      else:
        result=Results(res)
      columns = []
      for result_column in result._searchable_result_columns():
        colName = result_column['name']
        colLabel = ''
        for s in colName.split('_'):
          colLabel += s.capitalize()
        try:
          colType = {'i':'int','t':'string'}[result_column['type']]
        except:
          _globals.writeException(self,'[query]: Unknown type '+str(result_column.get('type',None)))
        column = {}
        column['id'] = colName
        column['key'] = colName
        column['label'] = colLabel
        column['name'] = colLabel
        column['type'] = colType
        column['sort'] = 1
        columns.append(column)
      records = []
      for row in result:
        record = {}
        for column in columns:
          record[column['id']] = row[column['id']]
        records.append(record)
      return {'columns':columns,'records':records}


    # ------------------------------------------------------------------------------------------
    #  ZMSSqlDb.getEntitiesAndViews:
    # ------------------------------------------------------------------------------------------
    def getEntitiesAndViews(self):
      tables = self.getEntities()
      tables.extend(self.getViews())
      return tables


    # ------------------------------------------------------------------------------------------
    #  ZMSSqlDb.getEntities:
    # ------------------------------------------------------------------------------------------
    def getEntities(self):
      entities = []
      databaseAdptr = getattr(self,self.connection_id)
      for tableBrwsr in databaseAdptr.tpValues():
        tableName = tableBrwsr.Name().lower()
        tableType = tableBrwsr.Type().upper()
        if tableType == 'TABLE':
          columns = []
          for columnBrwsr in tableBrwsr.tpValues():
            colId = columnBrwsr.tpId().lower()
            colDescr = columnBrwsr.Description().upper()
            if colDescr.find('INTEGER')>=0:
              colType = 'int'
            elif colDescr.find('CHAR')>=0:
              colSize = 50
              i = colDescr.find('(')
              if i >= 0:
                j = colDescr.find(')')
                if j >= 0:
                  colSize = int(colDescr[i+1:j])
              if colSize > 50:
                colType = 'text'
              else:
                colType = 'string'
            else:
              colType = 'string'
            attrs = {}
            attrs['key'] = colId
            attrs['id'] = attrs['key']
            attrs['label'] = ''
            for s in colId.split('_'):
              attrs['label'] += s.capitalize()
            attrs['name'] = attrs['label']
            attrs['mandatory'] = colDescr.find('NOT NULL')>0
            attrs['type'] = colType
            attrs['sort'] = 1
            columns.append(attrs)
          entity = {}
          entity['id'] = tableName
          entity['sql'] = 'SELECT * FROM '+tableName
          entity['columns'] = columns
          entities.append((tableName,entity))
      entities.sort()
      return map(lambda x: x[1], entities)
      

    # ------------------------------------------------------------------------------------------
    #  ZMSSqlDb.getViews:
    # ------------------------------------------------------------------------------------------
    def getViews(self):
      views = []
      sql_views = getattr(self,'sql_views',{})
      for id in sql_views.keys():
        view = sql_views[id].copy()
        view['id'] = id
        views.append((id,view))
      views.sort()
      return map(lambda x: x[1], views)
      
      

    """
    ###############################################################################################    
    ###
    ###   R e c o r d S e t
    ###
    ###############################################################################################    
    """

    # ---------------------------------------------------------------------------------------------
    #  ZMSSqlDb.recordSet_Init:
    #
    #  Initializes RecordSet.
    # ---------------------------------------------------------------------------------------------
    def recordSet_Init(self, REQUEST):
      entities = self.getEntitiesAndViews()
      ids = map(lambda x: x['id'], entities)
      
      SESSION = REQUEST.SESSION      
      SESSION.set('qentity',REQUEST.form.get('qentity',SESSION.get('qentity',ids[0])))
      if SESSION['qentity'] not in ids:
        SESSION.set('qentity',ids[0])

      entity = filter(lambda x: x['id']==SESSION['qentity'], entities)[0]
      result = self.query(entity['sql'])
      res = result['records']
      REQUEST.set('grid_cols',result['columns'])
      REQUEST.set('res',res)
      
      return res


    # ---------------------------------------------------------------------------------------------
    #  ZMSSqlDb.recordSet_Filter:
    #
    #  Filters RecordSet.
    # ---------------------------------------------------------------------------------------------
    def recordSet_Filter(self, REQUEST):
      entities = self.getEntitiesAndViews()
      
      SESSION = REQUEST.SESSION      
      SESSION.set('qfilters',REQUEST.form.get('qfilters',SESSION.get('qfilters',1)))
      res = REQUEST['res']

      for i in range(SESSION['qfilters']):
        filterattr='filterattr%i'%i
        filtervalue='filtervalue%i'%i
        sessionattr='%s_%s'%(filterattr,self.id)
        sessionvalue='%s_%s'%(filtervalue,self.id)
        
        #-- Set filter parameters in Session
        if REQUEST.get('action','')=='':
          if REQUEST.get('btn','')==self.getLangStr('BTN_RESET',REQUEST['manage_lang']):
            SESSION.set(sessionattr,'')
            SESSION.set(sessionvalue,'')
          elif REQUEST.get('btn','')==self.getLangStr('BTN_REFRESH',REQUEST['manage_lang']):
            SESSION.set(sessionattr,REQUEST.form.get(filterattr,''))
            SESSION.set(sessionvalue,REQUEST.form.get(filtervalue,''))
        
        #-- Apply filter parameters 
        entity = filter(lambda x: x['id']==SESSION['qentity'], entities)[0]
        if entity.has_key('columns'):
          for attr in entity['columns']:
            if attr.get('name','')!='':
              if SESSION.get(sessionattr,'')==attr['id'] and SESSION.get(sessionvalue,'')!='':
                if attr['type'] in ['float','int']:
                  res = self.filter_list(res,attr['id'],self.formatObjAttrValue(attr,SESSION.get(sessionvalue,''),REQUEST['lang']))
                else:
                  res = self.filter_list(res,attr['id'],SESSION.get(sessionvalue,''))
      
      REQUEST.set('res',res)
      
      return res


    # ---------------------------------------------------------------------------------------------
    #  ZMSSqlDb.recordSet_Sort:
    #
    #  Sorts RecordSet.
    # ---------------------------------------------------------------------------------------------
    def recordSet_Sort(self, REQUEST):
      entities = self.getEntitiesAndViews()
      
      SESSION = REQUEST.SESSION      
      res=REQUEST['res']
      qorder=REQUEST.get('qorder','')
      qorderdir=REQUEST.get('qorderdir','asc')

      entity = filter(lambda x: x['id']==SESSION['qentity'], entities)[0]
      if entity.has_key('columns'):
        for attr in entity['columns']:
          if qorder=='':
            if attr.get('name','')!='' and attr.get('custom','')!='':
              qorder = attr['id']
              if attr.get('type','') in ['date','datetime']:
                qorderdir = 'desc'
      res = self.sort_list(res,qorder,qorderdir)
      
      REQUEST.set('res',res)
      REQUEST.set('qorder',qorder)
      REQUEST.set('qorderdir',qorderdir)
      
      return res


    """
    ###############################################################################################    
    ###
    ###   P r o p e r t i e s
    ###
    ###############################################################################################    
    """

    ###############################################################################################
    #  ZMSSqlDb.manage_expertProperties: 
    #
    #  Change Sql-Database expert properties.
    ###############################################################################################
    def manage_expertProperties(self, lang, manage_lang, btn, REQUEST, RESPONSE): 
      """ ZMSSqlDb.manage_expertProperties """
      message = ''
      id = REQUEST.get('id','')

      # Insert.
      # -------
      if btn == self.getLangStr('BTN_INSERT',manage_lang):
        id = REQUEST.get('newId').strip()
        sql = REQUEST.get('newSql')
        setSqlView(self,None,id,sql)
        message = self.getLangStr('MSG_INSERTED',manage_lang)%id
      
      # Change.
      # -------
      elif btn == self.getLangStr('BTN_CHANGE',manage_lang):
        oldId = id
        id = REQUEST.get('inpId').strip()
        sql = REQUEST.get('inpSql')
        setSqlView(self,oldId,id,sql)
        message = self.getLangStr('MSG_CHANGED',manage_lang)
      
      # Delete.
      # -------
      elif btn == self.getLangStr('BTN_DELETE',manage_lang):
        delSqlView(self,id)
        id = ''
        message = self.getLangStr('MSG_CHANGED',manage_lang)
      
      # Return with message.
      message = urllib.quote(message)
      return RESPONSE.redirect('manage_mainExpert?id=%s&lang=%s&manage_lang=%s&manage_tabs_message=%s'%(id,lang,manage_lang,message))


    ###############################################################################################
    #  ZMSSqlDb.manage_changeProperties: 
    #
    #  Change Sql-Database properties.
    ###############################################################################################
    def manage_changeProperties(self, lang, manage_lang, REQUEST=None, RESPONSE=None): 
      """ ZMSSqlDb.manage_changeProperties """
      message = ''

      # Change.
      # -------
      if REQUEST.get('btn','') != self.getLangStr('BTN_CANCEL',manage_lang):
        self.connection_id = REQUEST['connection_id']
        message = self.getLangStr('MSG_CHANGED',manage_lang)
      
      # Return with message.
      message = urllib.quote(message)
      return RESPONSE.redirect('manage_properties?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(lang,manage_lang,message))


    # ---------------------------------------------------------------------------------------------
    #  ZMSSqlDb.isPage
    # ---------------------------------------------------------------------------------------------
    def isPage(self): return 0

    # ---------------------------------------------------------------------------------------------
    #  ZMSSqlDb.isPageElement
    # ---------------------------------------------------------------------------------------------
    def isPageElement(self): return 0

    # ---------------------------------------------------------------------------------------------
    #  ZMSSqlDb.getTitlealt
    # ---------------------------------------------------------------------------------------------
    def getTitlealt(self,REQUEST): return self.display_type(REQUEST)


    """
    ###############################################################################################
    ###
    ###  H T M L - P r e s e n t a t i o n 
    ###
    ###############################################################################################
    """

    # ---------------------------------------------------------------------------------------------
    #	ZMSSqlDb._getBodyContent:
    #
    #	HTML presentation of Textarea. 
    # ---------------------------------------------------------------------------------------------
    def _getBodyContent(self, REQUEST, thumbnail=0):
      # Render HTML presentation.
      text = '<h2>%s</h2>'%self.display_type(REQUEST)
      text += '<p class="description">'
      c = 0
      for entity in self.getEntities():
        if c > 0: text += '<br />'
        text += '- '+entity['id']
        c += 1
      text += '</p>'
      # Return <html>-presentation.
      return text

    # ---------------------------------------------------------------------------------------------
    #	ZMSSqlDb.renderShort:
    #
    #	Renders short presentation of Textarea.
    # ---------------------------------------------------------------------------------------------
    def renderShort(self, REQUEST):
      return self._getBodyContent(REQUEST,thumbnail=1)

###################################################################################################
